@@ -319,15 +319,14 @@ function playerPaused(player) {
 }
 
 function setPlayerActive(player) {
+    pendingActivePlayer = player;
+
     if (isNaN(player.duration)) {
         // Ignore this player for now until we know a duration
         // In durationchange event handler we'll check for this and end up here again
-        pendingActivePlayer = player;
         return;
     }
 
-    pendingActivePlayer = undefined;
-
     // Ignore short sounds, they are most likely a chat notification sound
     // A stream has a duration of Infinity
     // Note that "NaN" is also not finite but we already returned earlier for that
@@ -335,6 +334,7 @@ function setPlayerActive(player) {
         return;
     }
 
+    pendingActivePlayer = undefined;
     activePlayer = player;
 
     // when playback starts, send along metadata
@@ -510,8 +510,17 @@ function registerPlayer(player) {
     players.push(player);
 }
 
+function findAllPlayersFromNode(node) {
+    if (typeof node.getElementsByTagName !== "function") {
+        return [];
+    }
+
+    return [...node.getElementsByTagName("video"), ...node.getElementsByTagName("audio")];
+}
+
+
 function registerAllPlayers() {
-    var players = document.querySelectorAll("video,audio");
+    var players = findAllPlayersFromNode(document);
     players.forEach(registerPlayer);
 }
 
@@ -557,12 +566,12 @@ function loadMpris() {
     var observer = new MutationObserver(function (mutations) {
         mutations.forEach(function (mutation) {
             mutation.addedNodes.forEach(function (node) {
-                if (typeof node.matches !== "function" || typeof node.querySelectorAll !== "function") {
+                if (typeof node.matches !== "function") {
                     return;
                 }
 
                 // Check whether the node itself or any of its children is a player
-                var players = Array.from(node.querySelectorAll("video,audio"));
+                var players = findAllPlayersFromNode(node);
                 if (node.matches("video,audio")) {
                     players.unshift(node);
                 }
@@ -573,12 +582,12 @@ function loadMpris() {
             });
 
             mutation.removedNodes.forEach(function (node) {
-                if (typeof node.matches !== "function" || typeof node.querySelectorAll !== "function") {
+                if (typeof node.matches !== "function") {
                     return;
                 }
 
                 // Check whether the node itself or any of its children is the current player
-                var players = Array.from(node.querySelectorAll("video,audio"));
+                var players = findAllPlayersFromNode(node);
                 if (node.matches("video,audio")) {
                     players.unshift(node);
                 }
@@ -603,11 +612,6 @@ function loadMpris() {
         });
     });
 
-    observer.observe(document, {
-        childList: true,
-        subtree: true
-    });
-
     window.addEventListener("pagehide", function () {
         // about to navigate to a different page, tell our extension that the player will be gone shortly
         // we listen for tab closed in the extension but we don't for navigating away as URL change doesn't
@@ -622,6 +626,11 @@ function loadMpris() {
 
     document.addEventListener("DOMContentLoaded", function() {
         registerAllPlayers();
+
+        observer.observe(document, {
+            childList: true,
+            subtree: true
+        });
     });
 
     document.addEventListener("fullscreenchange", () => {
@@ -698,27 +707,100 @@ function loadMediaSessionsShim() {
 
         executeScript(`
             function() {
-                ${mediaSessionsClassName} = function() {};
-                ${mediaSessionsClassName}.callbacks = {};
-                ${mediaSessionsClassName}.metadata = null;
-                ${mediaSessionsClassName}.playbackState = "none";
-                ${mediaSessionsClassName}.sendMessage = function(action, payload) {
-                    let event = new CustomEvent("pbiMprisMessage", {
-                        detail: {
-                            action: action,
-                            payload: payload
+                ${mediaSessionsClassName}_constructor = function() {
+                    this.callbacks = {};
+                    this.pendingCallbacksUpdate = 0;
+                    this.metadata = null;
+                    this.playbackState = "none";
+
+                    this.sendMessage = function (action, payload) {
+                        let event = new CustomEvent("pbiMprisMessage", {
+                            detail: {
+                                action: action,
+                                payload: payload
+                            }
+                        });
+                        window.dispatchEvent(event);
+                    };
+
+                    this.executeCallback = function (action) {
+                        let details = {
+                            action: action
+                            // for seekforward, seekbackward, seekto there's additional information one would need to add
+                        };
+                        this.callbacks[action](details);
+                    };
+
+                    this.setCallback = function (name, cb) {
+                        const oldCallbacks = Object.keys(this.callbacks).sort();
+
+                        if (cb) {
+                            this.callbacks[name] = cb;
+                        } else {
+                            delete this.callbacks[name];
                         }
-                    });
-                    window.dispatchEvent(event);
-                };
-                ${mediaSessionsClassName}.executeCallback = function (action) {
-                    let details = {
-                        action: action
-                        // for seekforward, seekbackward, seekto there's additional information one would need to add
+
+                        const newCallbacks = Object.keys(this.callbacks).sort();
+
+                        if (oldCallbacks.toString() === newCallbacks.toString()) {
+                            return;
+                        }
+
+                        if (this.pendingCallbacksUpdate) {
+                            return;
+                        }
+
+                        this.pendingCallbacksUpdate = setTimeout(() => {
+                            this.pendingCallbacksUpdate = 0;
+
+                            // Make sure to send the current callbacks, not "newCallbacks" at the time of starting the timeout
+                            const callbacks = Object.keys(this.callbacks);
+                            this.sendMessage("callbacks", callbacks);
+                        }, 0);
+                    };
+
+                    this.setMetadata = function (metadata) {
+                        // MediaMetadata is not a regular Object so we cannot just JSON.stringify it
+                        let newMetadata = {};
+
+                        let dirty = (!metadata != !this.metadata);
+                        if (metadata) {
+                            const keys = Object.getOwnPropertyNames(Object.getPrototypeOf(metadata));
+
+                            const oldMetadata = this.metadata || {};
+
+                            keys.forEach((key) => {
+                                const value = metadata[key];
+                                if (!value || typeof value === "function") {
+                                    return; // continue
+                                }
+
+                                // We only have Strings or the "artwork" Array, so a toString() comparison should suffice...
+                                dirty |= (value.toString() !== (oldMetadata[key] || "").toString());
+
+                                newMetadata[key] = value;
+                            });
+                        }
+
+                        this.metadata = metadata;
+
+                        if (dirty) {
+                            this.sendMessage("metadata", newMetadata);
+                        }
+                    };
+
+                    this.setPlaybackState = function (playbackState) {
+                        if (this.playbackState === playbackState) {
+                            return;
+                        }
+
+                        this.playbackState = playbackState;
+                        this.sendMessage("playbackState", playbackState);
                     };
-                    this.callbacks[action](details);
                 };
 
+                ${mediaSessionsClassName} = new ${mediaSessionsClassName}_constructor();
+
                 if (!navigator.mediaSession) {
                     navigator.mediaSession = {};
                 }
@@ -727,12 +809,7 @@ function loadMediaSessionsShim() {
 
                 var oldSetActionHandler = navigator.mediaSession.setActionHandler || noop;
                 navigator.mediaSession.setActionHandler = function (name, cb) {
-                    if (cb) {
-                        ${mediaSessionsClassName}.callbacks[name] = cb;
-                    } else {
-                        delete ${mediaSessionsClassName}.callbacks[name];
-                    }
-                    ${mediaSessionsClassName}.sendMessage("callbacks", Object.keys(${mediaSessionsClassName}.callbacks));
+                    ${mediaSessionsClassName}.setCallback(name, cb);
 
                     // Call the original native implementation
                     // "call()" is needed as the real setActionHandler is a class member
@@ -743,32 +820,15 @@ function loadMediaSessionsShim() {
                 };
 
                 Object.defineProperty(navigator.mediaSession, "metadata", {
-                    get: function() { return ${mediaSessionsClassName}.metadata; },
-                    set: function(newValue) {
-                        ${mediaSessionsClassName}.metadata = newValue;
-
-                        // MediaMetadata is not a regular Object so we cannot just JSON.stringify it
-                        var newMetadata = {};
-                        if (newValue) {
-                            var keys = Object.getOwnPropertyNames(Object.getPrototypeOf(newValue));
-
-                            keys.forEach(function (key) {
-                                var value = newValue[key];
-                                if (typeof value === "function") {
-                                    return; // continue
-                                }
-                                newMetadata[key] = newValue[key];
-                            });
-                        }
-
-                        ${mediaSessionsClassName}.sendMessage("metadata", newMetadata);
+                    get: () => ${mediaSessionsClassName}.metadata,
+                    set: (newValue) => {
+                        ${mediaSessionsClassName}.setMetadata(newValue);
                     }
                 });
                 Object.defineProperty(navigator.mediaSession, "playbackState", {
-                    get: function() { return ${mediaSessionsClassName}.playbackState; },
-                    set: function(newValue) {
-                        ${mediaSessionsClassName}.playbackState = newValue;
-                        ${mediaSessionsClassName}.sendMessage("playbackState", newValue);
+                    get: () => ${mediaSessionsClassName}.playbackState,
+                    set: (newValue) => {
+                        ${mediaSessionsClassName}.setPlaybackState(newValue);
                     }
                 });
 
