@@ -33,7 +33,9 @@ class DownloadPlugin : public AbstractBrowserPlugin
     Q_OBJECT
 public:
     explicit DownloadPlugin(QObject *parent);
+    bool onLoad() override;
     bool onUnload() override;
+    using AbstractBrowserPlugin::handleData;
     void handleData(const QString &event, const QJsonObject &data) override;
 private:
     QHash<int, DownloadJob *> m_jobs;
