@@ -31,6 +31,8 @@
 static const QString s_muteTab = QStringLiteral("mute");
 static const QString s_unmuteTab = QStringLiteral("unmute");
 
+K_EXPORT_PLASMA_RUNNER_WITH_JSON(TabsRunner, "plasma-runner-browsertabs.json")
+
 TabsRunner::TabsRunner(QObject *parent, const QVariantList &args)
     : Plasma::AbstractRunner(parent, args)
 {
@@ -147,7 +149,8 @@ void TabsRunner::match(Plasma::RunnerContext &context)
                 {QStringLiteral("service"), service},
                 {QStringLiteral("tabId"), tabId},
                 {QStringLiteral("audible"), audible},
-                {QStringLiteral("muted"), muted}
+                {QStringLiteral("muted"), muted},
+                {QStringLiteral("url"), url}
             };
 
             Plasma::QueryMatch match(this);
@@ -276,9 +279,12 @@ QDBusMessage TabsRunner::createMessage(const QString &service, const QString &me
 
 QMimeData *TabsRunner::mimeDataForMatch(const Plasma::QueryMatch &match)
 {
-    Q_UNUSED(match);
-    // TODO return tab url or maybe for firefox a magic "dragging tab off a window" mime?
-    return nullptr;
+    const QVariantHash tabData = match.data().toHash();
+    const QUrl url = tabData.value(QStringLiteral("url")).toUrl();
+
+    auto *data = new QMimeData();
+    data->setUrls({url});
+    return data;
 }
 
 QList<QAction *> TabsRunner::actionsForMatch(const Plasma::QueryMatch &match)
@@ -301,6 +307,4 @@ QList<QAction *> TabsRunner::actionsForMatch(const Plasma::QueryMatch &match)
     return actions;
 }
 
-K_EXPORT_PLASMA_RUNNER(browsertabs, TabsRunner)
-
 #include "tabsrunner.moc"
